@@ -135,8 +135,6 @@ hd_desktop_launch_banner_show (HDWM *hdwm, HDWMWatchableApp *app, gpointer data)
                      APP_LAUNCH_BANNER_MSG_LOADING),
                      lapp_name ? _(lapp_name) : "" );
 
-  g_debug ("Launching banner %s...",info->msg);
-
   info->banner = GTK_WIDGET (hildon_banner_show_animation (NULL, NULL, info->msg));
 
   g_timeout_add (interval, hd_desktop_launch_banner_timeout, info);
@@ -500,9 +498,12 @@ hd_desktop_system_conf_dir_changed (GnomeVFSMonitorHandle *handle,
 
   if (!g_ascii_strcasecmp (filename, HD_DESKTOP_CONFIG_FILE))
   {
+    /* Disabling desktop conf file monitoing to avoid crashes for now */
+#if 0
     g_free (priv->config_file);
     priv->config_file = hd_desktop_get_conf_file_path (HD_DESKTOP_CONFIG_FILE);
     hd_desktop_load_containers (desktop);
+#endif
   } else {
     HDDesktopContainerInfo *info;
     GList *plugin_list = NULL;
@@ -550,9 +551,12 @@ hd_desktop_user_conf_dir_changed (GnomeVFSMonitorHandle *handle,
 
   if (!g_ascii_strcasecmp (filename, HD_DESKTOP_CONFIG_FILE))
   {
+    /* Disabling desktop conf file monitoing to avoid crashes for now */
+#if 0
     g_free (priv->config_file);
     priv->config_file = hd_desktop_get_conf_file_path (HD_DESKTOP_CONFIG_FILE);
     hd_desktop_load_containers (desktop);
+#endif
   } else {
     HDDesktopContainerInfo *info;
     GList *plugin_list = NULL;
@@ -657,297 +661,6 @@ hd_desktop_watch_dir (gchar                 *plugin_dir,
                           user_data);
 }
 
-#if 0
-#ifdef HAVE_LIBOSSO
-static gint 
-hildon_desktop_rpc_cb (const gchar *interface,
-                       const gchar *method,
-                       GArray *arguments,
-                       gpointer data,
-                       osso_rpc_t *retval)
-{
-  HDDesktop *desktop;
-  osso_rpc_t *val[5];
-  gint i;
-
-  if (!interface || !method || !arguments || !data) 
-  {
-    return OSSO_ERROR;
-  }
-
-  desktop = (HDDesktop *) data;
-
-  for (i = 0; i < arguments->len; ++i) 
-  {
-    val[i] = &g_array_index (arguments, osso_rpc_t, i);
-  }
-
-  if (g_str_equal("system_note_infoprint", method))
-  {
-    if (arguments->len < 1 || val[0]->type != DBUS_TYPE_STRING ) 
-    {
-      if (arguments->len < 1) 
-      {
-        retval->value.s = g_strdup ("Not enough args to infoprint");
-      } 
-      else 
-      {
-        g_sprintf (retval->value.s,
-                   "Wrong type param to infoprint (%d)", val[0]->type);
-      }
-
-      g_warning (retval->value.s);
-
-      return OSSO_ERROR;
-    }
-
-    hildon_banner_show_information( NULL, NULL, val[0]->value.s);
-  }
-  else if (g_str_equal ("system_note_dialog", method))
-  {
-    if (arguments->len < 2 ||
-        val[0]->type != DBUS_TYPE_STRING ||
-        val[1]->type != DBUS_TYPE_INT32 ) 
-    {
-      if (arguments->len < 2) 
-      {
-        retval->value.s = "Not enough args to dialog";
-      } 
-      else 
-      {
-        retval->value.s = "Wrong type of arguments to dialog";
-      }
-
-      g_warning (retval->value.s);
-
-      return OSSO_ERROR;
-    }
-
-    hildon_status_bar_lib_prepare_dialog (val[1]->value.i, 
-                                          NULL,
-                                          val[0]->value.s,  
-                                          0, 
-                                          NULL, 
-                                          NULL);
-  }
-  else if (g_str_equal ("open_closeable_system_dialog", method))
-  {
-    gint id;
-    const gchar *btext = NULL;
-
-    if (arguments->len < 4 ||
-        val[0]->type != DBUS_TYPE_STRING ||
-        val[1]->type != DBUS_TYPE_INT32 ||
-        val[2]->type != DBUS_TYPE_STRING ||
-        val[3]->type != DBUS_TYPE_BOOLEAN)
-    {
-      retval->type = DBUS_TYPE_STRING;
-
-      if (arguments->len < 4) 
-      {
-        retval->value.s = g_strdup ("Not enough args to dialog");
-      } 
-      else 
-      {
-        retval->value.s = g_strdup ("Wrong type of arguments to dialog");
-      }
-
-      g_warning (retval->value.s);
-
-      return OSSO_ERROR;
-    }
-
-    if ((val[2]->value.s)[0] != '\0')
-    {
-      btext = val[2]->value.s;
-    }
-
-    id = hildon_status_bar_lib_open_closeable_dialog (val[1]->value.i,
-                                                      val[0]->value.s, 
-                                                      btext, 
-                                                      val[0]->value.b);
-
-    retval->type = DBUS_TYPE_INT32;
-    retval->value.i = id;
-  }
-  else if (g_str_equal ("close_closeable_system_dialog", method))
-  {
-    /* The id of the dialog is given as argument */
-    if (arguments->len < 1 || val[0]->type != DBUS_TYPE_INT32)
-    {
-      retval->type = DBUS_TYPE_STRING;
-
-      if (arguments->len < 1) 
-      {
-        retval->value.s = g_strdup ("Not enough args to dialog");
-      } 
-      else 
-      {
-        retval->value.s = g_strdup ("Argument has invalid type");
-      }
-
-      g_warning (retval->value.s);
-
-      return OSSO_ERROR;
-    }
-
-    hildon_status_bar_lib_close_closeable_dialog (val[0]->value.i);
-  }
-  else if (g_str_equal( "get_system_dialog_response", method))
-  {
-    gint response = -1;
-
-    /* The id of the dialog is given as argument */
-    if (arguments->len < 1 || val[0]->type != DBUS_TYPE_INT32)
-    {
-      retval->type = DBUS_TYPE_STRING;
-
-      if (arguments->len < 1) 
-      {
-        retval->value.s = g_strdup ("Not enough args to dialog");
-      } 
-      else 
-      {
-        retval->value.s = g_strdup ("Argument has invalid type");
-      }
-
-      g_warning (retval->value.s);
-
-      return OSSO_ERROR;
-    }
-
-    response = hildon_status_bar_lib_get_dialog_response (val[0]->value.i);
-
-    retval->type = DBUS_TYPE_INT32;
-    retval->value.i = response;
-  }
-  else if (g_str_equal ("delayed_infobanner", method))
-  {
-      gint parent_window_id = 0;
-
-      if (arguments->len < 4 ||
-          arguments->len > 5 ||
-          val[0]->type != DBUS_TYPE_INT32 ||
-          val[1]->type != DBUS_TYPE_INT32 ||
-          val[2]->type != DBUS_TYPE_INT32 ||
-          val[3]->type != DBUS_TYPE_STRING ||
-          (arguments->len == 5 && val[4]->type != DBUS_TYPE_INT32))
-      {
-        if (arguments->len < 4) 
-        {
-          retval->value.s = "Not enough arguments.";
-        } 
-        else 
-        {
-          g_sprintf (retval->value.s, "Wrong type of arguments: "
-                     "(%d,%d,%d,%d); was expecting (%d, %d, %d and %d)",
-                     val[0]->type, val[1]->type,
-                     val[2]->type, val[3]->type,
-                     DBUS_TYPE_INT32, DBUS_TYPE_INT32,
-                     DBUS_TYPE_INT32, DBUS_TYPE_STRING);
-        }
-
-        g_warning (retval->value.s);
-
-        return OSSO_ERROR;
-      }
-
-      if (arguments->len == 5)
-        parent_window_id = val[4]->value.i;
-
-      return _delayed_infobanner_add (val[0]->value.i,
-                                      val[1]->value.i,
-                                      val[2]->value.i,
-                                      val[3]->value.s,
-                                      parent_window_id);
-  }
-  else if (g_str_equal( "cancel_delayed_infobanner", method))
-  {
-    if (arguments->len > 1 ||
-        val[0]->type != DBUS_TYPE_INT32 ) 
-    {
-      retval->type = DBUS_TYPE_STRING;
-
-      if (arguments->len > 1) 
-      {
-        retval->value.s = "Too many args.";
-      } 
-      else 
-      {
-        g_sprintf(retval->value.s, "Wrong type of arguments: "
-                  "(%d), was expecting int (%d)",
-                  val[0]->type, DBUS_TYPE_INT32);
-      }
-
-      g_warning (retval->value.s);
-
-      return OSSO_ERROR;
-    }
-
-    _delayed_infobanner_remove (GINT_TO_POINTER (val[0]->value.i));
-
-    /* This function returns boolean for the timeout functions, *
-     * we don't care about that here. It's false always, anyway */
-    return OSSO_OK;
-
-  }
-  else if( g_str_equal( "statusbar_insensitive", method ))
-  {
-    sb_is_sensitive = FALSE;
-
-    gtk_container_foreach (GTK_CONTAINER (panel->fixed),
-                           (GtkCallback) (statusbar_insensitive_cb),
-                           NULL);
-
-    gtk_container_foreach (GTK_CONTAINER (panel->arrow_button),
-                           (GtkCallback) (statusbar_insensitive_cb),
-                           NULL);
-
-    return OSSO_OK;
-  }
-  else if (g_str_equal("statusbar_sensitive", method))
-  {
-    sb_is_sensitive = TRUE;
-
-    gtk_container_foreach (GTK_CONTAINER (panel->fixed),
-                           (GtkCallback) (statusbar_sensitive_cb),
-                           NULL);
-
-    gtk_container_foreach (GTK_CONTAINER (panel->arrow_button),
-                           (GtkCallback) (statusbar_sensitive_cb),
-                           NULL);
-
-    return OSSO_OK;
-  }
-  else if (g_str_equal("statusbar_get_conditional", method))
-  {
-    int i;
-
-    for (i = 0; i < HSB_MAX_NO_OF_ITEMS; i++) /* Can we break earlier? */
-    {
-       if (panel->items[i])
-       {
-         statusbar_send_signal (osso_get_dbus_connection (panel->osso),
-                                HILDON_STATUS_BAR_ITEM (panel->items[i]),
-                                hildon_status_bar_item_get_conditional
-                                (HILDON_STATUS_BAR_ITEM (panel->items[i])));
-       }
-    }
-
-    return OSSO_OK;
-  }
-  else
-  {
-    g_warning ("Unknown SB RPC method");
-
-    return OSSO_ERROR;
-  }
-
-  return OSSO_OK;
-}
-#endif
-#endif
-
 static void 
 hd_desktop_load_containers (HDDesktop *desktop)
 {
@@ -1122,8 +835,6 @@ hd_desktop_load_containers (HDDesktop *desktop)
                                       "padding-top",    padding_top,
                                       "padding-bottom", padding_bottom,
                                       NULL);
-
-      hildon_home_window_applets_init (HILDON_HOME_WINDOW (info->container));
     }
     else if (!g_ascii_strcasecmp (type, HD_CONTAINER_TYPE_PANEL_BOX) ||
  	     !g_ascii_strcasecmp (type, HD_CONTAINER_TYPE_PANEL_EXPANDABLE))
@@ -1345,31 +1056,14 @@ hd_desktop_create_note_infoprint (const gchar *summary,
 
   return banner;
 }
-	
+
 static void
-hd_desktop_system_notification_dialog_response (GtkWidget *widget,
-	       					gint response,	
-		                                HDDesktopNotificationInfo *ninfo)
+hd_desktop_show_next_system_dialog (HDDesktop *desktop)
 {
-  HildonDesktopNotificationManager *nm;
-  HDDesktop *desktop;
   GtkWidget *next_dialog = NULL;
 
-  desktop = ninfo->desktop;
-
-  nm = (HildonDesktopNotificationManager *) 
-	  gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (desktop->priv->nm));
-
-  hildon_desktop_notification_manager_close_notification (nm, ninfo->id, NULL);
-  
-  g_free (ninfo);
-
   g_queue_pop_head (desktop->priv->dialog_queue);
-  gtk_widget_destroy (widget);
 
-  g_debug ("DIALOG QUEUE: %d", g_queue_get_length (desktop->priv->dialog_queue));
-    
-  /* Show next system notification dialog if present */
   while (!g_queue_is_empty (desktop->priv->dialog_queue))
   {
     next_dialog = (GtkWidget *) g_queue_peek_head (desktop->priv->dialog_queue);
@@ -1386,17 +1080,92 @@ hd_desktop_system_notification_dialog_response (GtkWidget *widget,
   }
 }
 
+static void
+hd_desktop_system_notification_dialog_destroy (GtkWidget *widget, HDDesktop *desktop)
+{
+  hd_desktop_show_next_system_dialog (desktop);
+}
+	
+static void
+hd_desktop_system_notification_dialog_response (GtkWidget *widget,
+	       					gint response,	
+		                                HDDesktopNotificationInfo *ninfo)
+{
+  HildonDesktopNotificationManager *nm;
+  HDDesktop *desktop;
+
+  desktop = ninfo->desktop;
+
+  nm = (HildonDesktopNotificationManager *) 
+	  gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (desktop->priv->nm));
+
+  hildon_desktop_notification_manager_call_action (nm, ninfo->id, "default");
+  hildon_desktop_notification_manager_close_notification (nm, ninfo->id, NULL);
+  
+  g_free (ninfo);
+
+  gtk_widget_destroy (widget);
+}
+
+static gboolean
+hd_desktop_pulsate_progress_bar (gpointer user_data)
+{
+  if (GTK_IS_PROGRESS_BAR (user_data)) 
+  {
+    gtk_progress_bar_pulse (GTK_PROGRESS_BAR (user_data));
+    return TRUE;
+  }
+  else
+  {
+    return FALSE;
+  }
+}
+
 static GtkWidget *
 hd_desktop_create_note_dialog (const gchar *summary, 
 			       const gchar *body, 
-			       const gchar *icon_name)
+			       const gchar *icon_name,
+			       gint         dialog_type,
+			       gchar      **actions)
 {
   GtkWidget *note;
+  gint i;
+  
+  /* If it's a progress dialog, add the progress bar */
+  if (dialog_type == 4)
+  {
+    GtkWidget *progressbar;
 
-  note = hildon_note_new_information_with_icon_name (NULL, 
-		  				     body, 
-						     icon_name);
+    progressbar = gtk_progress_bar_new ();
 
+    gtk_progress_bar_pulse (GTK_PROGRESS_BAR (progressbar));
+
+    note = hildon_note_new_cancel_with_progress_bar (NULL,
+                                                     body,
+                                                     GTK_PROGRESS_BAR (progressbar));
+
+    g_timeout_add (100, hd_desktop_pulsate_progress_bar, progressbar);
+  }
+  else
+  {
+    note = hildon_note_new_information_with_icon_name (NULL, 
+  		  				       body, 
+  						       icon_name);
+  }
+  
+  /* If there's a default action, get the label and set
+   * the button text */
+  for (i = 0; actions && actions[i] != NULL; i += 2)
+  {
+    gchar *label = actions[i + 1];
+    
+    if (g_str_equal (actions[i], "default"))
+    {
+      hildon_note_set_button_text (HILDON_NOTE (note), label);
+      break;
+    }
+  }
+  
   return note;
 }
 
@@ -1428,6 +1197,7 @@ hd_desktop_system_notification_received (GtkTreeModel *model,
   GtkWidget *notification = NULL;
   GHashTable *hints;
   GValue *hint;
+  gchar **actions;
   const gchar *hint_s;
   gchar *summary;
   gchar *body;
@@ -1444,6 +1214,7 @@ hd_desktop_system_notification_received (GtkTreeModel *model,
 		      HD_NM_COL_SUMMARY, &summary,
 		      HD_NM_COL_BODY, &body,
 		      HD_NM_COL_ICON_NAME, &icon_name,
+		      HD_NM_COL_ACTIONS, &actions,
 		      HD_NM_COL_HINTS, &hints,
 		      -1);
 
@@ -1461,10 +1232,16 @@ hd_desktop_system_notification_received (GtkTreeModel *model,
   else if (g_str_equal (hint_s, "system.note.dialog")) 
   {
     HDDesktopNotificationInfo *ninfo;
-
+    gint dialog_type = 0;
+    
+    hint = g_hash_table_lookup (hints, "dialog-type");
+    dialog_type = g_value_get_int (hint);
+    
     notification = hd_desktop_create_note_dialog (summary, 
 		    				  body, 
-						  icon_name);
+						  icon_name,
+						  dialog_type,
+						  actions);
 
     ninfo = g_new0 (HDDesktopNotificationInfo, 1); 
 
@@ -1476,6 +1253,11 @@ hd_desktop_system_notification_received (GtkTreeModel *model,
   		      G_CALLBACK (hd_desktop_system_notification_dialog_response),
   		      ninfo);
 
+    g_signal_connect (G_OBJECT (notification),
+  		      "destroy",
+  		      G_CALLBACK (hd_desktop_system_notification_dialog_destroy),
+  		      desktop);
+
     if (g_queue_is_empty (desktop->priv->dialog_queue))
     {
       gtk_widget_show_all (notification);
